@@ -1,8 +1,8 @@
 """
 Author: Damien GUEHO
-Copyright: Copyright (C) 2022 Damien GUEHO
+Copyright: Copyright (C) 2023 Damien GUEHO
 License: Public Domain
-Version: 23
+Version: 24
 Date: April 2022
 Python: 3.7.7
 """
@@ -12,15 +12,15 @@ import numpy as np
 from numpy import linalg as LA
 
 
-def observerKalmanIdentificationAlgorithm(input_signal, output_signal, **kwargs):
+def observerKalmanIdentificationAlgorithm(input_signals, output_signals, **kwargs):
     """
         Purpose:
             Compute the coefficients :math:`h_i`, called system Markov parameters, of the weighting sequence description
             :math:`\\boldsymbol{y}_k = CA^k\\boldsymbol{x}_0 + \displaystyle\sum_{i=0}^kh_i\\boldsymbol{u}_{k-i}`.
 
         Parameters:
-            - **input_signal** (``DiscreteSignal``): the input signal.
-            - **output_signal** (``DiscreteSignal``): the output signal.
+            - **input_signals** (``DiscreteSignal``): the input signals.
+            - **output_signals** (``DiscreteSignal``): the output signals.
             - **number_of_parameters** (``int``, optional): number :math:`d` of system Markov parameters to consider as non-zero in the \
             weighting sequence description. If not specified, **number_of_parameters = output_signal.number_steps**.
             - **stable_order** (``int``, optional): the order :math:`d'` such that :math:`CA^{d'}\\boldsymbol{x}_0 \\simeq 0`. If not specified, \
@@ -89,30 +89,46 @@ def observerKalmanIdentificationAlgorithm(input_signal, output_signal, **kwargs)
 
         """
 
-    # Get data from Signals
-    y = output_signal.data
-    u = input_signal.data
+    number_signals = len(input_signals)
 
     # Get dimensions
-    input_dimension = input_signal.dimension
+    input_dimension = input_signals[0].dimension
+    output_dimension = output_signals[0].dimension
 
     # Get number of Markov parameters to compute
-    number_steps = output_signal.number_steps
+    number_steps = output_signals[0].number_steps
     number_of_parameters = min(kwargs.get('number_of_parameters', number_steps), number_steps)
     stable_order = kwargs.get('stable_order', 0)
 
-    # Build matrix U
-    U = np.zeros([input_dimension * number_of_parameters, number_steps - stable_order])
-    for i in range(0, number_of_parameters):
-        U[i * input_dimension:(i + 1) * input_dimension, max(0, i - stable_order):number_steps - stable_order] = u[:, stable_order - min(i, stable_order):number_steps - i]
+    Us = []
+    ys = []
+    yy = np.zeros([output_dimension, (number_steps - stable_order) * number_signals])
 
-    print(U)
-    # Get Y
-    Y = np.matmul(y[:, stable_order:], LA.pinv(U))
+    for k in range(number_signals):
+        # Get data from Signals
+        y = output_signals[k].data
+        u = input_signals[k].data
+
+        # Build matrix U
+        U = np.zeros([input_dimension * number_of_parameters, (number_steps - stable_order)])
+        for i in range(0, number_of_parameters):
+            U[i * input_dimension:(i + 1) * input_dimension, max(0, i - stable_order):number_steps - stable_order] = u[:, stable_order - min(i, stable_order):number_steps - i]
+        Us.append(U)
+
+        # Build matrix yy
+        ys.append(y[:, stable_order:])
+
+    # Ravel
+    UU = np.stack(Us, 2).reshape(Us[0].shape[0], -1)
+    yy = np.stack(ys, 2).reshape(ys[0].shape[0], -1)
+
+    # Get observer Markov parameters
+    Y = np.matmul(yy, LA.pinv(UU))
+    print('Error OKID =', LA.norm(yy - np.matmul(Y, UU)))
 
     # Get Markov parameters
     markov_parameters = [Y[:, 0:input_dimension]]
     for i in range(number_of_parameters - 1):
         markov_parameters.append(Y[:, i * input_dimension + input_dimension:(i + 1) * input_dimension + input_dimension])
 
-    return markov_parameters, U, y[:, stable_order:]
+    return markov_parameters, UU, yy
