@@ -1,8 +1,8 @@
 """
 Author: Damien GUEHO
-Copyright: Copyright (C) 2022 Damien GUEHO
+Copyright: Copyright (C) 2023 Damien GUEHO
 License: Public Domain
-Version: 23
+Version: 24
 Date: April 2022
 Python: 3.7.7
 """
@@ -69,23 +69,34 @@ def timeVaryingEigenSystemRealizationAlgorithmFromInitialConditionResponse(free_
 
 
     # Construct Y
-    Y = np.zeros([(p + 1) * output_dimension, number_free_decay_experiments, number_steps - p])
-    for k in range(number_steps - p):
-        for i in range(p + 1):
-            for j in range(number_free_decay_experiments):
-                Y[i * output_dimension:(i + 1) * output_dimension, j, k] = free_decay_experiments.output_signals[j].data[:, i + k]
+    Y = np.zeros([number_steps * output_dimension, number_free_decay_experiments])
+    for j in range(number_free_decay_experiments):
+        Y[:, j] = free_decay_experiments.output_signals[j].data.reshape(number_steps * output_dimension)
+
+
+    # # Construct Y
+    # Y = np.zeros([(p + 1) * output_dimension, number_free_decay_experiments, number_steps - p])
+    # for k in range(number_steps - p):
+    #     print(k)
+    #     for i in range(p + 1):
+    #         for j in range(number_free_decay_experiments):
+    #             Y[i * output_dimension:(i + 1) * output_dimension, j, k] = free_decay_experiments.output_signals[j].data[:, i + k]
+
+
+    ## Calculating for first time step
+    (R1, sigma1, St1) = LA.svd(Y[0:p * output_dimension, :], full_matrices=True)
 
 
     # Calculating A and C matrices - Free Response
     for k in range(number_steps - p):
         print(k)
         # SVD Y1
-        (R1, sigma1, St1) = LA.svd(Y[0:p * output_dimension, :, k], full_matrices=True)
+        # (R1, sigma1, St1) = LA.svd(Y[0:p * output_dimension, :, k], full_matrices=True)
         Sigma1 = np.diag(sigma1)
         Sigma.append(sigma1)
 
         # SVD Y2
-        (R2, sigma2, St2) = LA.svd(Y[output_dimension:(p + 1) * output_dimension, :, k], full_matrices=True)
+        (R2, sigma2, St2) = LA.svd(Y[output_dimension + k:(p + 1) * output_dimension + k, :], full_matrices=True)
         Sigma2 = np.diag(sigma2)
 
         if mac_and_msv:
@@ -129,6 +140,9 @@ def timeVaryingEigenSystemRealizationAlgorithmFromInitialConditionResponse(free_
         A_id[:, :, k] = np.matmul(X2, LA.pinv(X1))
         C_id[:, :, k] = O1[0:output_dimension, :]
 
+        # Time shift
+        (R1, sigma1, St1) = (R2, sigma2, St2)
+
 
     # Create corresponding functions
     def A(tk):
